﻿using System;
using System.Data;
using NHibernate.SqlTypes;
using NHibernate.Type;

namespace NHibernate.Test   
{
    /// <summary>
    /// Maps a <see cref="System.DateTime" /> Property to a Time column that only stores the Hours, Minutes, 
    /// and Seconds of the DateTime.
    /// Also you have for <see cref="DbType.Time"/> handling, the NHibernate Type <see cref="TimeAsTimeSpanType"/>,
    /// the which maps to a <see cref="TimeSpan"/>.
    /// </summary>
    /// <remarks>
    /// <para>
    /// This class provides a similar functionality to the <see cref="NHibernate.Type.TimeType" />, but passes a 
    /// <see cref="System.TimeSpan"/> parameter to the DB command in the Set() method, which is the correct type for 
    /// the underlying <see cref="DbType.Time"/> (for SQL Server, at least) according to the MSDN documentation on 
    /// data type mappings: http://msdn.microsoft.com/en-us/library/cc716729.aspx.
    /// </para>
    /// <para>
    /// A more appropriate choice to store the duration/time is the <see cref="TimeSpanType"/>.
    /// The underlying <see cref="DbType.Time"/> tends to be handled differently by different
    /// DataProviders.
    /// </para>
    /// </remarks>
    [Serializable]
    public class TimeType : PrimitiveType, IIdentifierType
    {
        private static readonly DateTime BaseDateValue = new DateTime(1753, 01, 01);

        public TimeType()
            : base(SqlTypeFactory.Time)
        {
        }

        public override string Name
        {
            get { return "Time"; }
        }

        public override object Get(IDataReader rs, int index)
        {
            try
            {
                if (rs[index] is TimeSpan) //For those dialects where DbType.Time means TimeSpan.
                {
                    TimeSpan time = (TimeSpan)rs[index];
                    return BaseDateValue.AddTicks(time.Ticks);
                }

                DateTime dbValue = Convert.ToDateTime(rs[index]);
                return new DateTime(1753, 01, 01, dbValue.Hour, dbValue.Minute, dbValue.Second);
            }
            catch (Exception ex)
            {
                throw new FormatException(string.Format("Input string '{0}' was not in the correct format.", rs[index]), ex);
            }
        }

        public override object Get(IDataReader rs, string name)
        {
            return Get(rs, rs.GetOrdinal(name));
        }

        public override System.Type ReturnedClass
        {
            get { return typeof(DateTime); }
        }

        public override void Set(IDbCommand st, object value, int index)
        {
            DateTime time = (DateTime)value;
            ((IDataParameter)st.Parameters[index]).Value = new TimeSpan(time.Hour, time.Minute, time.Second);
        }

        public override bool IsEqual(object x, object y)
        {
            if (x == y)
            {
                return true;
            }
            if (x == null || y == null)
            {
                return false;
            }

            DateTime date1 = (DateTime)x;
            DateTime date2 = (DateTime)y;
            if (date1.Equals(date2))
                return true;

            return date1.Hour == date2.Hour
                         && date1.Minute == date2.Minute
                         && date1.Second == date2.Second;
        }

        public override int GetHashCode(object x, EntityMode entityMode)
        {
            DateTime date = (DateTime)x;
            int hashCode = 1;
            unchecked
            {
                hashCode = 31 * hashCode + date.Second;
                hashCode = 31 * hashCode + date.Minute;
                hashCode = 31 * hashCode + date.Hour;
            }
            return hashCode;
        }

        public override string ToString(object val)
        {
            return ((DateTime)val).ToShortTimeString();
        }

        public object StringToObject(string xml)
        {
            return string.IsNullOrEmpty(xml) ? null : FromStringValue(xml);
        }

        public override object FromStringValue(string xml)
        {
            return DateTime.Parse(xml);
        }

        public override System.Type PrimitiveClass
        {
            get { return typeof(DateTime); }
        }

        public override object DefaultValue
        {
            get { return BaseDateValue; }
        }

        public override string ObjectToSQLString(object value, Dialect.Dialect dialect)
        {
            return "'" + ((DateTime)value).ToShortTimeString() + "'";
        }
    }
}
